#!/bin/sh

PKG_STR_WIZARD_UPGRADE_TITLE="Uppdatera WebDAV Server"
PKG_STR_WIZARD_UPGRADE_DESC="WebDAV Server 2.4.0 och senare versioner har inte längre stöd för kalenderfunktionen. Se <a href=\\\"https://www.synology.com/knowledgebase/DSM/tutorial/File_Sharing/How_to_migrate_calendar_from_WebDAV_Server_to_Synology_Calendar\\\" target=\\\"_blank\\\">den här artikeln</a> om du vill migrera kalendrarna till Synology Calendar."
PKG_STR_WIZARD_UPGRADE_NOTE="Observera att alla .ics-filer sparas i sina ursprungliga mappar efter uppdateringen."
check_version_older() # $1 base version $2 target version
{
	local BASE_VER=$1
	local TARGET_VER=$2

	# if no base ver, always reture false
	if [ -z "${BASE_VER}" ]; then
		return 0;
	fi

	# getting major, minor, build
	local target_major=`echo ${TARGET_VER} | sed 's/^\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\).*/\1/'`
	local target_minor=`echo ${TARGET_VER} | sed 's/^\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\).*/\2/'`
	local target_maintenance=`echo ${TARGET_VER} | sed 's/^\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\).*/\3/'`
	local target_build=`echo ${TARGET_VER} | sed 's/^\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\).*/\4/'`
	target_build=$(echo ${target_build} | sed 's/^0*//')
	target_build=$((target_build % 10000))

	local BASE_VER_NUM=`echo "$BASE_VER" | awk 'BEGIN {FS="."} {print NF}'`
	if [ ${BASE_VER_NUM} -eq 2 ]; then
		local base_major=`echo ${BASE_VER} | sed 's/^\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\).*/\1/'`
		local base_minor=`echo ${BASE_VER} | sed 's/^\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\).*/\2/'`
		local base_build=`echo ${BASE_VER} | sed 's/^\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\).*/\3/'`
	else
		local base_major=`echo ${BASE_VER} | sed 's/^\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\).*/\1/'`
		local base_minor=`echo ${BASE_VER} | sed 's/^\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\).*/\2/'`
		local base_maintenance=`echo ${BASE_VER} | sed 's/^\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\).*/\3/'`
		local base_build=`echo ${BASE_VER} | sed 's/^\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\).*/\4/'`
	fi
	base_build=$(echo ${base_build} | sed 's/^0*//')
	base_build=$((base_build % 10000))

	# compare major, version must equal or above limitation
	if [ $target_major -lt $base_major ]; then
		return 1;
	elif [ $target_major -gt $base_major ]; then
		return 0;
	fi
	# compare minor
	if [ $target_minor -lt $base_minor ]; then
		return 1;
	elif [ $target_minor -gt $base_minor ]; then
		return 0;
	fi
	# compare mminor
	if [ ${BASE_VER_NUM} -ne 2 ]; then
		if [ $target_maintenance -lt $base_maintenance ]; then
			return 1;
		elif [ $target_maintenance -gt $base_maintenance ]; then
			return 0;
		fi
	fi
	# compare build
	if [ $target_build -lt $base_build ]; then
		return 1;
	else
		return 0;
	fi
}

CALDAV_EOL_VERSION="2.4.0-0039"
CALDAV_EOL_PAGE=`/bin/cat<<EOF
{
	"step_title": "${PKG_STR_WIZARD_UPGRADE_TITLE}",
	"items": [{
		"desc": "${PKG_STR_WIZARD_UPGRADE_DESC}"
	}, {
		"desc": "${PKG_STR_WIZARD_UPGRADE_NOTE}"
    }]
}
EOF`

main()
{
	local UpgradePages=""

    check_version_older ${SYNOPKG_OLD_PKGVER} ${CALDAV_EOL_VERSION}
    if [ 0 -eq $? ]; then
        UpgradePages="${CALDAV_EOL_PAGE}"
    fi

/bin/cat > ${SYNOPKG_TEMP_LOGFILE} <<EOF
[${UpgradePages}]
EOF

	exit 0
}

main "$@"
